package com.nisum.products_producer.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.nisum.products_producer.exception.KafkaPublishingException;
import com.nisum.products_producer.model.Product;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.core.reactive.ReactiveKafkaProducerTemplate;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import static org.springframework.kafka.support.KafkaHeaders.TOPIC;

@Service
public class KafkaProducerService {

    private static final Logger LOGGER = LoggerFactory.getLogger(KafkaProducerService.class);
    private final ReactiveKafkaProducerTemplate<String, String> kafkaProducerTemplate;
    private final ObjectMapper objectMapper;

    @Value("${spring.kafka.topic.product}")
    private String topic;



    public KafkaProducerService(ReactiveKafkaProducerTemplate<String, String> kafkaProducerTemplate, ObjectMapper objectMapper) {
        this.kafkaProducerTemplate = kafkaProducerTemplate;
        this.objectMapper = objectMapper;
    }

    public Mono<Void> sendProductToKafka(Product productRequest) {
        return Mono.justOrEmpty(productRequest)
                .switchIfEmpty(Mono.error(new IllegalArgumentException("ProductRequest cannot be null")))
                .flatMap(request -> {
                    try {
                        String productJson = objectMapper.writeValueAsString(request);
                        ProducerRecord<String, String> record = new ProducerRecord<>(topic, productJson);

                        return kafkaProducerTemplate.send(record)
                                .doOnSuccess(senderResult ->
                                        LOGGER.info(" Message sent successfully to Kafka: topic={}, offset={}, partition={}",
                                                topic, senderResult.recordMetadata().offset(), senderResult.recordMetadata().partition())
                                )
                                .doOnError(error ->
                                        LOGGER.error(" Failed to send message to Kafka: {}", error.getMessage(), error)
                                )
                                .then(); // Converts to Mono<Void>
                    } catch (JsonProcessingException e) {
                        return Mono.error(new KafkaPublishingException("Error converting ProductRequest to JSON", e));
                    }
                });
    }
}
