package com.nisum.products_producer.service;
import com.nisum.products_producer.dto.ProductDto;
import com.nisum.products_producer.exception.DuplicateProductException;
import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.repository.ProductRepository;
import com.nisum.products_producer.util.AppUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class ProductServiceImpl implements ProductService {

    private final ProductRepository productRepository;
    private static final Logger logger = LoggerFactory.getLogger(ProductServiceImpl.class);

    public ProductServiceImpl(ProductRepository productRepository) {
        this.productRepository = productRepository;
    }

    @Override
    public Mono<Product> saveProduct(Mono<Product> productMono) {
        logger.info("Received request to save product.");
        return productMono
                .flatMap(product -> productRepository.save(product)) // Save the product
                .doOnSuccess(saved -> logger.info("Product saved successfully: {}", saved))
                .onErrorResume(DuplicateKeyException.class, ex -> {
                    logger.error("Duplicate product detected: {}", ex.getMessage());
                    return Mono.error(new DuplicateProductException("Product already exists with the same name and price!"));
                })
                .doOnError(ex -> logger.error("Failed to save product: {}", ex.getMessage()));
    }


    @Override
    public Flux<Product> getAllProducts() {
        logger.info("Received request to fetch all products.");
        return productRepository.findAll()
                .doOnNext(product -> logger.info("Retrieved product: {}", product))
                .doOnComplete(() -> logger.info("Successfully fetched all products."))
                .doOnError(ex -> logger.error("Failed to fetch products: {}", ex.getMessage()));
    }
}
