package com.nisum.products_producer.controller;

import com.nisum.products_producer.advice.GlobalExceptionHandler;
import com.nisum.products_producer.dto.ProductDto;
import com.nisum.products_producer.exception.DuplicateProductException;
import com.nisum.products_producer.exception.InvalidProductNameException;
import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.service.ProductService;
import com.nisum.products_producer.service.ProductServiceImpl;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.reactive.WebFluxTest;

import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.test.web.reactive.server.WebTestClient;
import org.springframework.web.bind.support.WebExchangeBindException;
import reactor.core.publisher.Mono;

import java.util.Map;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.*;
import static org.assertj.core.api.Assertions.assertThat;

@WebFluxTest(ProductController.class)
@ExtendWith(MockitoExtension.class)
@Import(GlobalExceptionHandler.class)
class ProductControllerTest {

    private WebTestClient webTestClient;

    @MockBean
    private ProductService productService;

    @Autowired
    private ProductController productController;


    @BeforeEach
    void setUp() {

        productController = new ProductController(productService);

        webTestClient = WebTestClient.bindToController(productController)
                .controllerAdvice(new GlobalExceptionHandler())
                .build();
    }

    @Test
    public void addProductTest_Success(){

        // Given: A duplicate product DTO
        Product product = Product.builder()
                .id("105")
                .name("Laptop")
                .price(2500.00)
                .build();
        when(productService.saveProduct(any())).thenReturn(Mono.just(product));

        webTestClient.post()
                .uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(product)
                .exchange()
                .expectStatus().isCreated()
                .expectBody(Product.class)
                .consumeWith(response -> {
                    Product responseBody = response.getResponseBody();
                    assertNotNull(responseBody, "Response body should not be null");
                    assertEquals("105", responseBody.getId(), "Product ID mismatch");
                    assertEquals("Laptop", responseBody.getName(), "Product name mismatch");
                    assertEquals(2500.0, responseBody.getPrice(), "Product price mismatch");
                });

        // Verify that the service method was called once
        verify(productService, times(1)).saveProduct(any());
    }

    /*@Test
    void testSaveProduct_InvalidName_OnlyNumbers() {
        Product invalidProduct = new Product();
        invalidProduct.setId("102");
        invalidProduct.setName("12345"); // Only numbers
        invalidProduct.setPrice(2200.0);

        webTestClient.post()
                .uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(invalidProduct)
                .exchange()
                .expectStatus().isBadRequest();
    }*/
   /* @Test
    void testSaveProduct_InvalidName_OnlyNumbers() {
        Product invalidProduct = new Product();
        invalidProduct.setId("102");
        invalidProduct.setName("12345"); // Invalid name
        invalidProduct.setPrice(2200.0);

       *//* WebExchangeBindException validationException =
                new WebExchangeBindException(null, null);
        validationException.rejectValue("name", "Invalid name", "Name cannot be only numbers");

        when(productService.saveProduct(any())).thenReturn(Mono.error(validationException));*//*
        webTestClient.post().uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(invalidProduct)
                .exchange()
                .expectStatus().isBadRequest()
                .expectBody(String.class)
                .consumeWith(response ->
                        System.out.println("Response Body: " + response.getResponseBody())
                );
    }*/

    @Test
    public void testAddProduct_whenProductServiceFails_shouldReturnBadRequest() {
        // Arrange
        Product invalidProduct = Product.builder()
                .id("102")
                .name("1234")
                .price(2200.0)
                .build();

        //when(productService.saveProduct(any(Mono.class))).thenReturn(Mono.error(new RuntimeException("Service failure")));

        // Act & Assert
        webTestClient.post().uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .bodyValue(invalidProduct)
                .exchange()
                .expectStatus().isBadRequest()
                .expectBody()
                .jsonPath("$.name").isEqualTo("Product name must contain at least one letter and can include numbers, but cannot be only numbers.");
    }


    /*void testSaveProduct_DuplicateProduct() {
        ProductDto duplicateProduct = new ProductDto();
        duplicateProduct.setId("105");
        duplicateProduct.setName("Laptop");
        duplicateProduct.setPrice(2500.0);

        Mockito.when(productService.saveProduct(Mockito.any()))
                .thenReturn(Mono.error(new DuplicateProductException("Product already exists with the same name and price!")));

        // When: Sending a POST request to save the duplicate product
        webTestClient.post()
                .uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .body(Mono.just(duplicateProduct), ProductDto.class)
                .exchange()
                // Then: Expecting HTTP 409 Conflict (because of @ExceptionHandler)
                .expectStatus().isEqualTo(409)
                .expectBody()
                .consumeWith(response -> System.out.println("Response: " + new String(response.getResponseBody()))) // Print response for debugging
                .jsonPath("$").isEqualTo("Product already exists with the same name and price!");  // Validate error message
    }*/
    @Test
    void testSaveProduct_DuplicateProduct() {
        Product duplicateProduct = new Product();
        duplicateProduct.setId("105");
        duplicateProduct.setName("Laptop");
        duplicateProduct.setPrice(2500.0);

        /*Mockito.when(productService.saveProduct(Mockito.any()))
                .thenAnswer(invocation -> {
                    throw new DuplicateProductException("Product already exists with the same name and price!");
                });*/

        Mockito.when(productService.saveProduct(Mockito.any()))
                .thenThrow(new DuplicateProductException("Product already exists with the same name and price!"));

        webTestClient.post()
                .uri("/product/saveProduct")
                .contentType(MediaType.APPLICATION_JSON)
                .body(Mono.just(duplicateProduct), Product.class)
                .exchange()
                .expectStatus().isEqualTo(HttpStatus.CONFLICT) // Expect 409
                .expectBody()
                .jsonPath("$.message").isEqualTo("Product already exists with the same name and price!");
    }


    @Test
    void testDuplicateProductExceptionHandler() {
        GlobalExceptionHandler exceptionHandler = new GlobalExceptionHandler();
        ResponseEntity<Map<String, String>> response =
                exceptionHandler.handleDuplicateProductException(new DuplicateProductException("Duplicate Entry"));

        Assertions.assertEquals(HttpStatus.CONFLICT, response.getStatusCode());
        Assertions.assertEquals("Duplicate Entry", response.getBody().get("message"));
    }
}
