package com.nisum.products_producer.service;

import com.nisum.products_producer.dto.ProductDto;
import com.nisum.products_producer.model.Product;
import com.nisum.products_producer.repository.ProductRepository;
import com.nisum.products_producer.util.AppUtil;
import jakarta.validation.ConstraintViolationException;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.web.bind.support.WebExchangeBindException;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.test.StepVerifier;


import static org.hamcrest.Matchers.any;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class ProductServiceTest {


    @Mock
    private ProductRepository productRepository;

    @InjectMocks
    private ProductService productService;

    private ProductDto productDto;
    private Product product;

    @BeforeEach
    void setUp() {
        MockitoAnnotations.openMocks(this);
        productService = new ProductServiceImpl(productRepository); // Inject mock manually

        // Create Sample Product DTO
        product = new Product();
        product.setId("101");
        product.setName("iPhone 14");
        product.setPrice(999.99);
        // Convert DTO to Entity
      //  product = AppUtil.toEntity(productDto);
    }

    @Test
    void saveProductTest() {
        // Mock repository save behavior
        when(productRepository.save(Mockito.any(Product.class))).thenReturn(Mono.just(product));

        // Call Service Method
        Mono<Product> result = productService.saveProduct(Mono.just(product));

        // Verify Result using StepVerifier
        StepVerifier.create(result)
                .expectNextMatches(savedProduct ->
                        savedProduct.getId().equals("101") &&
                                savedProduct.getName().equals("iPhone 14") &&
                                savedProduct.getPrice().equals(999.99)
                )
                .verifyComplete();
    }

    @Test
    void testSaveProduct_FailsWhenNameIsNull() {
        Product product = new Product();
        product.setName(null);  // Invalid: Name is null
        product.setPrice(100.0);

        Mono<Product> productMono = Mono.just(product);

        StepVerifier.create(productService.saveProduct(productMono))
                .expectErrorMatches(throwable -> throwable instanceof NullPointerException)
                .verify();
    }

    @Test
    void testSaveProduct_FailsWhenNameIsOnlyNumbers() {
        Product product = new Product();
        product.setName("123456");  // Invalid: Name contains only numbers
        product.setPrice(200.0);

        Mono<Product> productMono = Mono.just(product);

        StepVerifier.create(productService.saveProduct(productMono))
                .expectErrorMatches(throwable -> throwable instanceof Exception)
                .verify();
    }



    @Test
    void getAllProductsTest() {
        // Mock repository findAll() behavior
        when(productRepository.findAll()).thenReturn(Flux.just(product));

        // Call Service Method
        Flux<Product> result = productService.getAllProducts();

        // Verify Result using StepVerifier
        StepVerifier.create(result)
                .expectNextMatches(product ->
                        product.getId().equals("101") &&
                                product.getName().equals("iPhone 14") &&
                                product.getPrice().equals(999.99)
                )
                .verifyComplete();

        // Verify Repository Interaction
        verify(productRepository, times(1)).findAll();
    }
}
